<?php
/*
 * Plugin Name: Envíos con Correo Argentino para Woocommerce
 * Plugin URI: https://correoarg.s3.amazonaws.com/plugins/wc-correoargentino.zip
 * Description: Integra WooCommerce con Correo Argentino para realizar envíos a todo el país.
 * Version: 1.0
 * Author: Correo Argentino
 * Author URI: https://correoargentino.zippin.app/
 * Requires PHP: 7
 * License: GPL2
 * License URI:  https://www.gnu.org/licenses/gpl-2.0.html
 * WC requires at least: 4.0.0
 * WC tested up to: 5.6.0
 * Text Domain: wcca
 */

if (!defined('ABSPATH')) {
    exit; // Exit if accessed directly.
}

define('WCCA_VERSION', '1.0');

define('WCCA_APIKEY', '');
define('WCCA_SECRETKEY', '');

define('WCCA_LOGGER_CONTEXT', serialize(array('source' => 'wcca')));

register_activation_hook(__FILE__, 'WCCA\CorreoArgentino\Utilities\activate_plugin');
add_filter('plugin_action_links_' . plugin_basename(__FILE__), 'WCCA\CorreoArgentino\Utilities\add_plugin_column_links');
add_filter('plugin_row_meta', 'WCCA\CorreoArgentino\Utilities\add_plugin_description_links', 10, 4);

require_once 'includes/method.php';
require_once 'includes/settings.php';
require_once 'includes/api.php';
require_once 'includes/functions.php';
require_once 'includes/utilities.php';

// Settings Page
add_action('admin_init', 'WCCA\CorreoArgentino\Settings\init_settings');
add_action('admin_menu', 'WCCA\CorreoArgentino\Settings\create_menu_option');

// Shipping Method
add_action('woocommerce_shipping_init', 'WCCA\CorreoArgentino\wcca_init');
add_filter('woocommerce_shipping_methods', 'WCCA\CorreoArgentino\Utilities\add_method');

// Checkout
add_action('woocommerce_checkout_update_order_meta', 'WCCA\CorreoArgentino\Utilities\update_order_meta');
add_filter('woocommerce_cart_shipping_method_full_label', 'WCCA\CorreoArgentino\Utilities\wcca_add_free_shipping_label', 10, 2);
add_filter('woocommerce_checkout_update_order_review', 'WCCA\CorreoArgentino\Utilities\clear_cache');

// Orders
add_action('woocommerce_order_status_changed', 'WCCA\CorreoArgentino\Utilities\process_order_status', 10, 3);
add_action('add_meta_boxes', 'WCCA\CorreoArgentino\Utilities\add_order_side_box');
add_filter('woocommerce_admin_order_actions', 'WCCA\CorreoArgentino\Utilities\add_action_button', 10, 2);
add_action('admin_enqueue_scripts', 'WCCA\CorreoArgentino\Utilities\add_button_css_file');

// Tracking shortcode
add_shortcode('wcca_tracking', 'WCCA\CorreoArgentino\Utilities\create_shortcode');

// Webhook handler
add_action('woocommerce_api_wcca', 'WCCA\CorreoArgentino\Utilities\handle_webhook');

// Compatibilidad con plugin Woocommerce Shipping Calculator On Product Page (Magerips)
add_filter( 'woocommerce_shipping_calculator_enable_city', '__return_true');